#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "base.h"
#include "log.h"
#include "buffer.h"

#include "plugin.h"
#include "stream.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/**
 * this is a ssi for a lighttpd plugin
 * 
 * just replaces every occurance of 'ssi' by your plugin name
 * 
 * e.g. in vim:
 * 
 *   :%s/ssi/myhandler/
 * 
 */



/* plugin config for all request/connections */

typedef struct {
	PLUGIN_DATA;
	array *conf_ssi_path;
} plugin_data;

/* init the plugin data */
INIT_FUNC(mod_ssi_init) {
	plugin_data *p;
	
	p = calloc(1, sizeof(*p));
	
	p->conf_ssi_path = array_init();
	
	return p;
}

/* detroy the plugin data */
FREE_FUNC(mod_ssi_free) {
	plugin_data *p = p_d;
	UNUSED(srv);
	
	if (!p) return HANDLER_GO_ON;
	
	array_free(p->conf_ssi_path);
	
	free(p);
	
	return HANDLER_GO_ON;
}

/* handle plugin config and check values */

SETDEFAULTS_FUNC(mod_ssi_set_defaults) {
	plugin_data *p = p_d;
	size_t i = 0;
	
	config_values_t cv[] = { 
		{ "ssi.path",                   NULL, T_CONFIG_ARRAY },       /* 0 */
		{ NULL,                         NULL, T_CONFIG_UNSET }
	};
	
	if (!p) return HANDLER_ERROR;
	
	/* 0 */
	cv[i++].destination = p->conf_ssi_path;
	
	if (0 != config_insert_values(srv, cv)) {
		return HANDLER_ERROR;
	}
	
	return HANDLER_GO_ON;
}

static int mod_ssi_handle_request(server *srv, connection *con, plugin_data *p) {
	stream s;
	off_t i;
	
	UNUSED(srv);
	UNUSED(p);
	
	/* get a stream to the file */
	
	if (-1 == stream_open(&s, con->physical.path)) {
		return -1;
	}
	
	
	/**
	 * <!--#element attribute=value attribute=value ... --> 
	 * 
	 * config 
	 *   errmsg 
	 *   sizefmt 
	 *   timefmt 
	 * echo 
	 *   var 
	 *   encoding 
	 * exec 
	 *   cgi 
	 *   cmd 
	 * fsize 
	 *   file 
	 *   virtual 
	 * flastmod 
	 * include 
	 *   file
	 *   virtual
	 * printenv 
	 * set
	 *   var 
	 *   value
	 * 
	 * if
	 * elif
	 * else
	 * endif
	 * 
	 * DATE_GMT 
	 *   The current date in Greenwich Mean Time. 
	 * DATE_LOCAL 
	 *   The current date in the local time zone. 
	 * DOCUMENT_NAME 
	 *   The filename (excluding directories) of the document requested by the user. 
	 * DOCUMENT_URI 
	 *   The (%-decoded) URL path of the document requested by the user. Note that in the case of nested include files, this is not then URL for the current document. 
	 * LAST_MODIFIED 
	 *   The last modification date of the document requested by the user. 
	 * USER_NAME 
	 *   Contains the owner of the file which included it.
	 * 
	 */
	
	/* parse file */
	for (i = 0; i < s.size; i++) {
		char *c = (char *)(s.start) + i;
		
		/**
		 * search for
		 * 
		 * <!--#element attribute=value attribute=value ... --> 
		 * 
		 */
		
		switch(*c) {
		case '<':
			
			if (s.size - i < 4) break;
			
			if (c[1] != '!' || 
			    c[2] != '-' ||
			    c[3] != '-' ||
			    c[4] != '#') break;
			
			c += 4;
			
			if (0 == strcmp(c, "flastmod ")) {
				
			}
			
			break;
		}
	}
	
	
	
	stream_close(&s);
	
	/* reset physical.path */
	
	buffer_reset(con->physical.path);
	
	return 0;
}

URIHANDLER_FUNC(mod_ssi_physical_path) {
	plugin_data *p = p_d;
	size_t k;
	
	if (con->physical.path->used == 0) return HANDLER_GO_ON;
	
	for (k = 0; k < p->conf_ssi_path->used; k++) {
		data_string *ds = (data_string *)p->conf_ssi_path->data[k];
		
		if (ds->value->used == 0) continue;
		
		if (buffer_is_equal_right_len(con->physical.path, ds->value, ds->value->used - 1)) {
			
			/* handle ssi-request */
			
			mod_ssi_handle_request(srv, con, p);
			
			return HANDLER_FINISHED;
		}
	}
	
	/* not found */
	return HANDLER_GO_ON;
}

/* this function is called at dlopen() time and inits the callbacks */

int mod_ssi_plugin_init(plugin *p) {
	p->name        = buffer_init_string("ssi");
	
	p->init        = mod_ssi_init;
	p->handle_physical_path = mod_ssi_physical_path;
	p->set_defaults  = mod_ssi_set_defaults;
	p->cleanup     = mod_ssi_free;
	
	p->data        = NULL;
	
	return 0;
}
